# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import NamedTuple
from towhee.operator import Operator, SharedType


class AddOperatorV2(Operator):
    """
    Add operator that can take in undefined amount of inputs and saves results to '<input_name>_sum'
    column of result dataframe.
    """

    def __init__(self, factor: int) -> None:
        super().__init__()
        self._factor = factor

    def __call__(self, **kwargs):
        cols = []
        ret = []
        for key, value in kwargs.items():
            cols.append((key + '_sum', type(value)))
            ret.append(value + self._factor)

        outputs = NamedTuple('Outputs', cols)
        return outputs(*ret) #pylint: disable=not-callable

    @property
    def shared_type(self):
        return SharedType.Shareable
