# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import NamedTuple
from towhee.operator import Operator
from numpy import linalg as LA
import numpy


class NormDataOperator(Operator):
    """
    Stateful operator
    """

    # TODO: init without argument
    def __init__(self, normed_num) -> None:
        super().__init__()
        self.normed_num = normed_num

    def __call__(self, data: numpy.ndarray) -> NamedTuple("Outputs", [("norm_data", numpy.ndarray)]):
        norm_embedding = data[0] / LA.norm(data[0])
        Outputs = NamedTuple("Outputs", [("norm_data", numpy.ndarray)])
        return Outputs(norm_embedding)
